@@ -2,7 +2,7 @@
 exports.localization = {};
 exports.localization.appName ={
   "en_US": "GeoGebra Classic",
-  "ar": "GeoGebra Classic",
+  "ar": "جيوجبرا كلاسيك",
   "bg": "ГеоГебра Класик",
   "bs": "GeoGebra Classic",
   "ca": "GeoGebra Clàssic",
