@@ -26,49 +26,69 @@ function printInfo(msg) {
 // Communication related code from GeoGebra to Electron.
 ipc = require('electron').ipcMain;
 
-// Giac evaluate, called in platform.js via ipc.
+// Keep a global reference of the window object, if you don't, the window will
+// be closed automatically when the JavaScript object is garbage collected.
+let win;
 
 // ipc calls from GeoGebra.
 ipc.on('log', function (event, message) {
-        event.returnValue = true;
-        if (!muteLogging) {
-            // Print fancy log message
-            console.log("\x1b[36mGeoGebra: \x1b[1m%s\x1b[0m", message);
-        }
-        if (logWatch) {
-            if (!logExitFound) {
-            if ((pos = message.search(logExit)) > 0)
-                logExitFound = true;
-                }
-            if (logExitFound) {
-                printDebug("Exiting due to matching log text");
-                process.exit(0);
-                }
-            }
-        if (getVersion) {
-            if (geoGebraVersion != "undef") {
-                process.exit(0);
-            }
-            if ((message.search("INFO") > 0) && ((pos = message.search("GeoGebra")) > 0))
-                geoGebraVersion = message.substring(pos);
-        }
+    event.returnValue = true;
+    if (!muteLogging) {
+        // Print fancy log message
+        console.log("\x1b[36mGeoGebra: \x1b[1m%s\x1b[0m", message);
     }
-);
-ipc.on('clipboard', function (event, data) {
-        event.returnValue = true;
-        const nativeImage = require('electron').nativeImage
-        img = nativeImage.createFromDataURL(data)
 
-        const {clipboard} = require('electron')
-        clipboard.write({image: img})
+    if (logWatch && message.search(logExit) > 0) {
+        logWatch = false;
+        printDebug("Exiting due to matching log text");
+        process.exit(0);
     }
-);
+
+    if (getVersion) {
+        if (geoGebraVersion != "undef") {
+            process.exit(0);
+        }
+        if ((message.search("INFO") > 0) && ((pos = message.search("GeoGebra")) > 0)) {
+            geoGebraVersion = message.substring(pos);
+        }
+    }
+});
 
 ipc.on('openUrl', function (event, url) {
-      const shell = require('electron').shell;
-      shell.openExternal(url);
-  }
-);
+    if (url.startsWith("https://")) {
+        const shell = require('electron').shell;
+        shell.openExternal(url);
+    }
+});
+
+const examState = {};
+
+ipc.on('exam', function(event, on) {
+    const { powerSaveBlocker } = require('electron');
+    if (on) {
+        trySpawn("disablekeys.exe", [], (proc) => {
+            examState.disablekeys = proc
+        });
+        win.setKiosk(true); // disables all system shortcuts on Mac and ESC on Windows
+        examState.psBlocker = powerSaveBlocker.start('prevent-display-sleep');
+        return;
+    } else {
+        win.setKiosk(false);
+
+        if (examState.disablekeys) {
+            examState.disablekeys.kill();
+        }
+        if (examState.psBlocker != undefined) {
+            powerSaveBlocker.stop(examState.psBlocker);
+            printDebug("Power saving restored");
+        }
+    }
+    printDebug("Exam mode: " + on);
+});
+
+ipc.on('unsaved', function(event, data) {
+    win.unsaved = JSON.parse(data);
+});
 
 const {app, BrowserWindow, Menu} = require('electron');
 const Config = require('electron-store');
@@ -79,10 +99,6 @@ const ggbConfig = require('./ggb-config.js');
 // we load 'windows-shortcuts' later only, when explicitly needed.
 // const ws = require('windows-shortcuts');
 
-// Keep a global reference of the window object, if you don't, the window will
-// be closed automatically when the JavaScript object is garbage collected.
-let win
-
 function createWindow(appArgs) {
 
     // Create the browser window...
@@ -94,7 +110,7 @@ function createWindow(appArgs) {
         webPreferences: {
             nodeIntegration: false,
             preload: __dirname + '/preload.js',
-            contextIsolation: false
+            contextIsolation: true // with this setting we need nativeWindowOpen:true (default)
         }
     };
     const config = new Config();
@@ -128,26 +144,25 @@ function createWindow(appArgs) {
     startUrl += "&lang=" + lang;
 
     win = new BrowserWindow(pref);
-    win.setMenuBarVisibility(false);
-    win.setAutoHideMenuBar(true);
+    win.removeMenu();
     win.loadURL(startUrl);
 
-	const appName = ggbConfig.appName || "classic";
+    const appName = ggbConfig.appName || "classic";
 
-	// startup ping (tracking usage statistics in Firebase)
-	var streamIDs = {
+    // startup ping (tracking usage statistics in Firebase)
+    var streamIDs = {
             "graphing": "G-Q0XVP4Q2QS",
             "cas": "G-WBR8KV3VDN",
             "classic": "G-P0Q2098X7G",
             "geometry": "G-6NZX4TXBMJ",
             "suite": "G-YTSRE0SSN4",
-	};
+    };
 
-	if (streamIDs[appName]) {
-		printDebug("fetching", "https://www.google-analytics.com/g/collect?v=2&tid=" + streamIDs[appName] + "&cid=$uid&uip=$uip&en=pageview&ep.origin=firebase");
-		fetch("https://www.google-analytics.com/g/collect?v=2&tid=" + streamIDs[appName] + "&cid=$uid&uip=$uip&en=pageview&ep.origin=firebase", 
-			{method: "POST", headers: {'User-Agent': 'GeoGebra Apps'}});
-	}
+    if (streamIDs[appName]) {
+        printDebug("fetching", "https://www.google-analytics.com/g/collect?v=2&tid=" + streamIDs[appName] + "&cid=$uid&uip=$uip&en=pageview&ep.origin=firebase");
+        fetch("https://www.google-analytics.com/g/collect?v=2&tid=" + streamIDs[appName] + "&cid=$uid&uip=$uip&en=pageview&ep.origin=firebase", 
+            {method: "POST", headers: {'User-Agent': 'GeoGebra Apps'}});
+    }
 
     // Open the DevTools.
     // win.webContents.openDevTools()
@@ -155,6 +170,11 @@ function createWindow(appArgs) {
         pref.show = true;
         win.show();
     });
+    win.webContents.on('did-create-window', (childWindow) => {
+       // hide menu in login window
+       childWindow.setMenuBarVisibility(false);
+       childWindow.setAutoHideMenuBar(true);
+    })
     // Emitted when the window is closed.
     win.on('closed', () => {
         printDebug("Window is closed");
@@ -179,8 +199,12 @@ function createWindow(appArgs) {
         // in an array if your app supports multi windows, this is the time
         // when you should delete the corresponding element.
         const {dialog} = require('electron');
-        const buttons = {message: win.unsaved[0], buttons: win.unsaved.slice(1)};
-        const option = (await dialog.showMessageBox(win, buttons)).response;
+        const dialogOptions = {
+                message: win.unsaved[0],
+                buttons: win.unsaved.slice(1),
+                title: 'GeoGebra'
+             };
+        const option = (await dialog.showMessageBox(win, dialogOptions)).response;
         if (option == 0) {
            win.loadURL("javascript:ggbApplet.checkSaved()");
         }
@@ -461,6 +485,7 @@ function validateUrl(url) {
         "google\\..*", "googleapis.com", "youtube.com", "googleusercontent.com", // google login
         "twitter.com", "twimg.com", // twitter login
         "facebook.com", "fbcdn.com", // FB login
+        ".*.hotjar.com",
         "live.com", "windows.net", "msauth.net", "microsoftonline.com"]; // Windows / Office 365 login
     return url && url.match(new RegExp('^https:\\/\\/([^/]*\\.)?(' + domains.join('|') + ')(\\/.*)$'));
 }
@@ -484,26 +509,6 @@ function enableFilter() {
 
 function localProtocol(request, callback) {
     const url = request.url.substr(5, request.url.length - 5);
-    if (url.match(/GGB_EXAM_ON/i)) {
-        trySpawn("disablekeys.exe", [], (proc) => {
-            global.disablekeys = proc
-        });
-        win.setKiosk(true); //disables all system shortcuts on Mac and ESC on Windows
-        return;
-    }
-    if (url.match(/GGB_EXAM_OFF/i)) {
-        win.setKiosk(false);
-
-        if (global.disablekeys) {
-            global.disablekeys.kill();
-        }
-    }
-    if (url.match(/SETUNSAVED/i)) {
-        var messagesJSON = decodeURIComponent(url.substring(url.indexOf("=") + 1));
-        win.unsaved = JSON.parse(messagesJSON);
-        return;
-    }
-
     const bits = url.split("?");
     const urlPath = bits[0];
     const normalized = path.normalize(`${__dirname}/${urlPath}`);
@@ -654,7 +659,6 @@ function parseCommandLineOptions() {
     ggbbase64 = "undef";
     forceWasm = false;
     logWatch = false;
-    logExitFound = false;
     forceGpu = true;
     var options = process.argv.length;
     var lastDetectedOption = 0;
